import unittest

from autohandshake import HandshakeBrowser, UserType
from autohandshake.src.exceptions import InvalidURLError, InvalidUserTypeError, \
    InsufficientPermissionsError
from autohandshake.src.constants import BASE_URL
from autohandshake.tests import TestSession
from enum import Enum


class TestHandshakeBrowserMainFunctionality(unittest.TestCase):

    def test_throws_error_if_given_invalid_url(self):
        completely_wrong_url = "https://www.google.com/"
        invalid_url = "https://jhu.joinhandhake.com/"
        not_a_url = "hello world"
        no_https = "jhu.joinhandshake.com"

        with self.assertRaises(InvalidURLError):
            browser = HandshakeBrowser()
            browser.get(completely_wrong_url)
            browser.quit()

        with self.assertRaises(InvalidURLError):
            browser = HandshakeBrowser()
            browser.get(invalid_url)
            browser.quit()

        with self.assertRaises(InvalidURLError):
            browser = HandshakeBrowser()
            browser.get(not_a_url)
            browser.quit()

        with self.assertRaises(InvalidURLError):
            browser = HandshakeBrowser()
            browser.get(no_https)
            browser.quit()

    def test_throws_error_for_insufficient_permissions(self):
        with self.assertRaises(InsufficientPermissionsError):
            with TestSession() as browser:
                browser.get(f'{BASE_URL}/schools/{int(browser.school_id) - 1}/edit')

    def test_throws_error_for_invalid_url_while_logged_in(self):
        with self.assertRaises(InvalidURLError):
            with TestSession() as browser:
                browser.get(f'{BASE_URL}/schools/abcdefg/edit')

    def test_element_exists_by_xpath(self):
        # tests valid as of 4/8/19
        browser = HandshakeBrowser()
        browser.get(BASE_URL)

        self.assertTrue(browser.element_exists_by_xpath('//div'))
        self.assertTrue(browser.element_exists_by_xpath("//a/span[@class='select2-chosen']"))

        self.assertFalse(browser.element_exists_by_xpath("//div[@class='this_isnt_a_real_class']"))

        browser.quit()


class TestHandshakeBrowserUserTypes(unittest.TestCase):
    """
    This test suite assumes the testing user has all three account types available to switch to.
    Additionally, each test assumes that the user is starting logged in as a career services
    user.
    """

    def test_sets_user_type_to_correct_type_upon_login(self):
        with TestSession() as browser:
            self.assertEqual(UserType.STAFF, browser.user_type)

    def test_switch_user_type_on_valid_page(self):
        with TestSession() as browser:
            self.assertEqual(UserType.STAFF, browser.user_type)
            browser.switch_users(UserType.EMPLOYER)
            self.assertEqual(UserType.EMPLOYER, browser.user_type)
            browser.switch_users(UserType.STUDENT)
            self.assertEqual(UserType.STUDENT, browser.user_type)
            browser.switch_users(UserType.STAFF)
            self.assertEqual(UserType.STAFF, browser.user_type)

    def test_browser_registers_correct_user_type_on_login(self):
        with TestSession() as browser:
            self.assertEqual(UserType.STAFF, browser.user_type)
            browser.switch_users(UserType.EMPLOYER)
        with TestSession() as browser:
            self.assertEqual(UserType.EMPLOYER, browser.user_type)
            browser.switch_users(UserType.STUDENT)
        with TestSession() as browser:
            self.assertEqual(UserType.STUDENT, browser.user_type)
            browser.switch_users(UserType.STAFF)
            self.assertEqual(UserType.STAFF, browser.user_type)

    def test_browser_throws_error_if_user_type_is_not_avaialble(self):
        class FakeUserType(Enum):
            FAKE_TYPE = 'not a type'

        with self.assertRaises(InvalidUserTypeError):
            with TestSession() as browser:
                self.assertEqual(UserType.STAFF, browser.user_type)
                browser.switch_users(FakeUserType.FAKE_TYPE)



from autohandshake.src.Pages.Page import Page
from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.constants import BASE_URL
from autohandshake.src.exceptions import NoSuchElementError, InsufficientPermissionsError


class StudentProfilePage(Page):
    """
    A student profile page in Handshake.
    """

    def __init__(self, student_id: int, browser: HandshakeBrowser):
        """
        :param student_id: the id of the student whose page to load
        :type student_id: int
        :param browser: a logged-in HandshakeBrowser
        :type browser: HandshakeBrowser
        """
        self._id = student_id
        super().__init__(f'{BASE_URL}/users/{student_id}', browser)

    @Page.require_user_type(UserType.STAFF)
    def view_as_student(self):
        view_as_btn_xpath = f'//a[@href="/users/{self._id}/view_as"]'
        try:
            self._browser.click_element_by_xpath(view_as_btn_xpath)
        except NoSuchElementError:
            raise InsufficientPermissionsError()
        self._browser.wait_until_element_is_clickable_by_xpath('//a[@href="/users/stop_viewing_as"]')
        self._browser.update_constants()

    def _validate_url(self, url):
        """
        Ensure that the given URL is a valid URL.

        Since the URL is not entered by the user, it is always valid.

        :param url: the url to validate
        :type url: str
        """
        return

    def _wait_until_page_is_loaded(self):
        """Wait until the page has finished loading."""
        self._browser.wait_until_element_exists_by_xpath(
            '//div[contains(@class, "student-profile-card student-profile-card__actions")]')



from autohandshake.src.exceptions import *
from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.HandshakeSession import HandshakeSession
from autohandshake.src.ViewAsStudent import ViewAsStudent
from autohandshake.src.Pages import AccessRequestPage, InsightsPage, \
    MajorSettingsPage, AppointmentCalendarPage, AppointmentTypePage, \
    AppointmentTypesListPage, InterviewSchedulePage, CareerInterestsPage, FileType, \
    RequestStatus, SurveyPage, WaitingRoomPage, EventsPage, LabelSettingsPage, \
    StaffPage



import unittest

from autohandshake import UserType, ViewAsStudent
from autohandshake.tests import TestSession
from autohandshake.src.constants import BASE_URL

TEST_USER_ID = 20474668  # my old fake student account


class TestViewAsStudent(unittest.TestCase):

    def test_view_as_student_changes_user_type_correctly(self):
        with TestSession() as browser:
            self.assertEqual(UserType.STAFF, browser.user_type)
            with ViewAsStudent(TEST_USER_ID, browser):
                self.assertEqual(UserType.STUDENT, browser.user_type)
                self.assertEqual(f'{BASE_URL}/users/{TEST_USER_ID}', browser.current_url)
            self.assertEqual(UserType.STAFF, browser.user_type)



import unittest
from autohandshake.src.Pages import StaffPage
from autohandshake.tests import TestSession


class TestStaffPage(unittest.TestCase):

    def test(self):
        with TestSession() as browser:
            staff_page = StaffPage(browser)
            names = staff_page.get_staff_names()
            self.assertEqual('Abby Jackson', names[0])
            self.assertEqual('Alayna Hayes', names[3])



from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.Pages import Page
from autohandshake.src.constants import BASE_URL


class CareerInterestsPage(Page):
    """A settings page for a single student's career interests.

    Only accessible while logged in as a student. Currently, the main utility
    of this class is allowing for the automated selection and deselection of
    Career Clusters.
    """

    def __init__(self, student_id: int, browser: HandshakeBrowser):
        """
        :param student_id: the id of the student whose career interests page you are to visit
        :type student_id: int
        :param browser: a logged-in HandshakeBrowser
        :type browser: HandshakeBrowser
        """
        self._id = student_id
        super().__init__(f'{BASE_URL}/users/{student_id}/career_interests', browser)
        browser.execute_script_on_element_by_xpath('window.scroll(0, 1000)')

    @Page.require_user_type(UserType.STUDENT)
    def cluster_is_selected_by_id(self, cluster_id: int) -> bool:
        """
        Check whether or not a specific cluster is selected, given the cluster ID.

        :param cluster_id: the id of the cluster to check
        :type cluster_id: int
        :return: True if the cluster is selected, false otherwise
        """
        cluster_xpath = f'//{self._get_cluster_xpath_by_id(cluster_id)}'
        return self._browser.element_is_selected_by_xpath(cluster_xpath)

    @Page.require_user_type(UserType.STUDENT)
    def cluster_is_selected_by_name(self, cluster_name: str) -> bool:
        """
        Check whether or not a specific cluster is selected, given the cluster name.

        :param cluster_name: the name of the cluster to check (case-sensitive)
        :type cluster_name: str
        :return: True if the cluster is selected, false otherwise
        """
        cluster_xpath = f'//{self._get_cluster_xpath_by_name(cluster_name)}'
        return self._browser.element_is_selected_by_xpath(cluster_xpath)

    @Page.require_user_type(UserType.STUDENT)
    def select_cluster_by_id(self, cluster_id: int):
        """
        Select a career cluster given its id.

        :param cluster_id: the id of the cluster to select
        :type cluster_id: int
        """
        cluster_xpath = f'//label[{self._get_cluster_xpath_by_id(cluster_id)}]'
        if not self.cluster_is_selected_by_id(cluster_id):
            self._browser.click_element_by_xpath(cluster_xpath)

    @Page.require_user_type(UserType.STUDENT)
    def select_cluster_by_name(self, cluster_name: str):
        """
        Select a career cluster given its name.

        :param cluster_name: the name of the cluster to select (case-sensitive)
        :type cluster_name: str
        """
        cluster_xpath = f'//label[{self._get_cluster_xpath_by_name(cluster_name)}]'
        if not self.cluster_is_selected_by_name(cluster_name):
            self._browser.click_element_by_xpath(cluster_xpath)

    @Page.require_user_type(UserType.STUDENT)
    def deselect_cluster_by_id(self, cluster_id: int):
        """
        Deselect a career cluster given its ID.

        :param cluster_id: the id of the cluster to deselect
        :type cluster_id: int
        """
        cluster_xpath = f'//label[{self._get_cluster_xpath_by_id(cluster_id)}]'
        if self.cluster_is_selected_by_id(cluster_id):
            self._browser.click_element_by_xpath(cluster_xpath)

    @Page.require_user_type(UserType.STUDENT)
    def deselect_cluster_by_name(self, cluster_name: str):
        """
        Deselect a career cluster given its name.

        :param cluster_name: the name of the cluster to deselect (case-sensitive)
        :type cluster_name: str
        """
        cluster_xpath = f'//label[{self._get_cluster_xpath_by_name(cluster_name)}]'
        if self.cluster_is_selected_by_name(cluster_name):
            self._browser.click_element_by_xpath(cluster_xpath)

    @Page.require_user_type(UserType.STUDENT)
    def save_interests(self):
        """
        Save any changes made to the student's career interests.

        You *must* call this method after making any changes to a student's
        career interests if you would like those changes to persist.
        """
        self._browser.click_element_by_xpath('//button[text()="Save My Career Interests"]')
        confirmation_msg_xpath = '//div[contains(text(), "Thanks for updating your Career Interests")]'
        self._browser.wait_until_element_exists_by_xpath(confirmation_msg_xpath)

    def _get_cluster_xpath_by_id(self, cluster_id: int) -> str:
        """
        Generate the xpath of a cluster's checkbox given its id.

        :param cluster_id: the id of the cluster to find
        :type cluster_id: int
        :return: the xpath of the cluster's check-box
        """
        return f'input[@value="{cluster_id}"]'

    def _get_cluster_xpath_by_name(self, cluster_name: str) -> str:
        """
        Generate the xpath of a cluster's checkbox given its name.

        :param cluster_name: the name of the cluster to find (case-sensitive)
        :type cluster_name: str
        :return: the xpath of the cluster's check-box
        """
        return f'input[../span[text()="{cluster_name}"]]'

    def _validate_url(self, url):
        """
        Ensure that the given URL is a valid URL.

        Since the URL is not entered by the user, it is always valid.

        :param url: the url to validate
        :type url: str
        """
        return

    def _wait_until_page_is_loaded(self):
        """Wait until the page has finished loading."""
        self._browser.wait_until_element_exists_by_xpath("//div[@data-bind='foreach: custom_job_interest_options']")



import configparser
import os
from cryptography.fernet import Fernet
from autohandshake import HandshakeSession
from autohandshake.src.constants import MAX_WAIT_TIME

config = configparser.ConfigParser()
config.read(os.path.dirname(__file__) + './config.ini')

password = Fernet(config['CONSTANTS']['KEY']).decrypt(bytes(config['CONSTANTS']['PASSWORD'],
                                                            encoding='utf-8')).decode("utf-8")
homepage = config['CONSTANTS']['HOMEPAGE']
email = config['CONSTANTS']['EMAIL']
school_id = config['CONSTANTS']['SCHOOL_ID']
download_dir = config['CONSTANTS']['DOWNLOAD_DIR']


class TestSession(HandshakeSession):
    """A testing instance of HandshakeSession that always logs in with config-based credentials"""

    download_dir = download_dir

    def __init__(self, max_wait_time=MAX_WAIT_TIME):
        super().__init__(homepage, email, password, max_wait_time)



from autohandshake.src.Pages.LoginPage import LoginPage
from autohandshake.src.Pages.Page import Page
from autohandshake.src.Pages.InsightsPage import InsightsPage, FileType
from autohandshake.src.Pages.AccessRequestPage import AccessRequestPage, RequestStatus
from autohandshake.src.Pages.AppointmentCalendarPage import AppointmentCalendarPage
from autohandshake.src.Pages.InterviewSchedulePage import InterviewSchedulePage
from autohandshake.src.Pages.CareerInterestsPage import CareerInterestsPage
from autohandshake.src.Pages.SurveyPage import SurveyPage
from autohandshake.src.Pages.WaitingRoomPage import WaitingRoomPage
from autohandshake.src.Pages.EventsPage import EventsPage

from autohandshake.src.Pages.SchoolSettings import *



class InvalidURLError(Exception):
    """Raised when a Page or Handshake session is passed an invalid Handshake URL"""
    pass


class WrongPageForMethodError(Exception):
    """Raised when a Page method is called when the browser is not on the correct page"""
    pass


class BrowserTimeoutError(Exception):
    """Raised when a HandshakeBrowser waits too long for something to happen"""
    pass


class NoSuchElementError(Exception):
    """Raised when a HandshakeBrowser attempts to find an element that doesn't exist"""
    pass


class InvalidEmailError(Exception):
    """Raised when attempting to log into Handshake with an invalid email"""
    pass


class InvalidPasswordError(Exception):
    """Raised when attempting to log into Handshake with an invalid password"""
    pass


class InsufficientPermissionsError(Exception):
    """Raised when HandshakeBrowser attempts to access a page for which the user has insufficient permissions"""
    pass


class InvalidUserTypeError(Exception):
    """Raised when the browser attempts to do something while logged in as the wrong user type"""
    pass



from autohandshake.src.Pages.Page import Page
from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.constants import BASE_URL
from autohandshake.src.exceptions import NoSuchElementError
from typing import List
from bs4 import BeautifulSoup
from dateutil import parser


class WaitingRoomPage(Page):
    """
    The Handshake appointment waiting room page.

    All office location filters are automatically cleared upon loading the page.
    """

    def __init__(self, browser: HandshakeBrowser):
        """
        :param browser: a logged-in HandshakeBrowser
        :type browser: HandshakeBrowser
        """
        super().__init__(f'{BASE_URL}/checkins/waiting_room', browser)
        self._clear_office_locations_filter()

    @Page.require_user_type(UserType.STAFF)
    def get_checkin_data(self) -> List[dict]:
        """
        Get student checkin data from the waiting room.

        Data is returned in the format:
        ::

            [
                {
                    'being_created': True,
                    'student_id': 2284230,
                    'student_name': 'John Ferguson',
                    'datetime': datetime.datetime(2018, 10, 31, 14, 33),
                    'office_location': 'Rogers Hall Room 302'
                },
                {
                    'being_created': False,
                    'student_id': 6574835,
                    'student_name': 'Alyssa Fernandez',
                    'datetime': datetime.datetime(2018, 11, 3, 9, 50),
                    'office_location': None
                }
            ]

        :return: a list of student checkin data dicts
        """
        check_ins = BeautifulSoup(self._browser.get_element_attribute_by_xpath(
            '//div[@data-bind="foreach: drop_ins"]', 'innerHTML'),
            'html.parser').find_all('div', recursive=False)
        return self._parse_checkins(check_ins)

    def _parse_checkins(self, checkins: List[BeautifulSoup]) -> List[dict]:
        result = []
        for checkin in checkins:
            result.append(self._parse_checkin(checkin))
        return result

    @staticmethod
    def _parse_checkin(checkin: BeautifulSoup) -> dict:
        result = {
            'being_created': checkin.find('a', {'class': 'btn'}).text == 'Being Created',
            'student_id': int(checkin.h3.a['href'][7:]),
            'student_name': checkin.h3.a.text,
            'datetime': parser.parse(checkin.h4.text[12:]),
            'office_location': checkin.select('h4:nth-of-type(2)')[0].text[17:]
        }
        if result['office_location'] == 'null':
            result['office_location'] = None
        return result

    @Page.require_user_type(UserType.STAFF)
    def _clear_office_locations_filter(self):
        """
        Clear any office location default search filters.
        """
        office_location_dropdown_xpath = '//li[@data-hook="office_locations-dropdown"]//a'
        all_options_btn_xpath = '//li[@data-hook="office_locations-dropdown"]//div[text()="All Options"]'
        modal_xpath = '//div[@id="filter-modal"]'
        done_btn_xpath = '//button[text()="Done"]'

        self._browser.click_element_by_xpath(office_location_dropdown_xpath)
        self._browser.click_element_by_xpath(all_options_btn_xpath)
        self._browser.wait_until_element_is_clickable_by_xpath(modal_xpath)
        self._uncheck_all_office_locations()
        self._browser.click_element_by_xpath(done_btn_xpath)
        self._browser.wait_until_element_does_not_exist_by_xpath(modal_xpath)

    def _uncheck_all_office_locations(self):
        i = 1
        while True:
            try:
                checkbox_xpath = f'//div[@class="not-selected"]/div[{i}]/input'
                office_location_xpath = f'//div[@class="not-selected"]/div[{i}]'
                if self._browser.element_is_selected_by_xpath(checkbox_xpath):
                    office_location = self._browser.get_element_attribute_by_xpath(
                        office_location_xpath, 'text')
                    self._browser.click_element_by_xpath(office_location_xpath)
                    self._wait_until_filter_is_removed(office_location)
                i += 1
            except NoSuchElementError:
                break

    def _wait_until_filter_is_removed(self, office_location):
        office_location_filter_card_xpath = self._get_filter_card_xpath(office_location)
        self._browser.wait_until_element_does_not_exist_by_xpath(office_location_filter_card_xpath)

    def _get_filter_card_xpath(self, office_location):
        return f'//div[@class="selected_filters"]//a[text()="{office_location}"]'

    def _validate_url(self, url):
        """
        Ensure that the given URL is a valid URL.

        Since the URL is not entered by the user, it is always valid.

        :param url: the url to validate
        :type url: str
        """
        return

    def _wait_until_page_is_loaded(self):
        """Wait until the page has finished loading."""
        self._browser.wait_until_element_exists_by_xpath('//div[@class="panel-title center"]/h4')



from autohandshake.src import (
    # main classes
    HandshakeSession,
    HandshakeBrowser,
    UserType,
    ViewAsStudent,

    # pages
    AccessRequestPage,
    AppointmentCalendarPage,
    InsightsPage,
    AppointmentTypesListPage,
    AppointmentTypePage,
    MajorSettingsPage,
    FileType,
    InterviewSchedulePage,
    CareerInterestsPage,
    SurveyPage,
    WaitingRoomPage,
    RequestStatus,
    EventsPage,
    LabelSettingsPage,
    StaffPage
)

from autohandshake.__about__ import (
    __author__,
    __copyright__,
    __email__,
    __version__
)


name = "autohandshake"



import unittest
from autohandshake import LabelSettingsPage
from autohandshake.tests import TestSession


class TestLabelSettingsPage(unittest.TestCase):

    def test_get_label_data_scrapes_webpage_correctly(self):
        with TestSession() as browser:
            label_settings_page = LabelSettingsPage(browser)
            labels = label_settings_page.get_label_data()
            self.assertEqual(['label_name', 'usage_counts', 'used_for', 'created_by_first_name', 'created_by_last_name',
                              'label_type'],
                             list(labels[0].keys()))

            # Warning: test data may change at any time, causing the test to fail!
            amm_expected = {
                'label_name': 'hwd: arts, media, and marketing academy',
                'usage_counts': {
                    'InterviewSchedule': 1,
                    'Job': 1,
                    'Event': 43
                },
                'used_for': 'All',
                'created_by_first_name': 'Krystle',
                'created_by_last_name': 'Wagemann',
                'label_type': 'normal',
            }
            ferpa_expected = {
                'label_name': 'ferpa',
                'usage_counts': {
                    'User': 842
                },
                'used_for': 'All',
                'created_by_first_name': None,
                'created_by_last_name': None,
                'label_type': 'system',
            }
            self.assertEqual(amm_expected, labels[34])
            self.assertEqual(ferpa_expected, labels[30])



from autohandshake.src.Pages import Page
from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.constants import BASE_URL
from bs4 import BeautifulSoup
from enum import Enum
from typing import List
from datetime import datetime
import math
import time
import re


class RequestStatus(Enum):
    """The possible access request statuses:

            * WAITING - requests that have not yet been processed
            * SUCCESSFUL - requests that were approved
            * REJECTED - requests that were rejected
            * FAILED - requests that failed for other reasons
            * ALL - include all request statuses
    """
    WAITING = 'Waiting'
    SUCCESSFUL = 'Successful'
    REJECTED = 'Rejected'
    FAILED = 'Failed'
    ALL = 'All'


class AccessRequestPage(Page):
    """
    The page on which users view and respond to Handshake access requests.

    Can only be accessed with a Career Services/admin-style account.
    """

    def __init__(self, browser: HandshakeBrowser):
        """
        :param browser: a logged-in HandshakeBrowser
        :type browser: HandshakeBrowser
        """
        super().__init__(f'{BASE_URL}/email_actions', browser)

    @Page.require_user_type(UserType.STAFF)
    def get_request_data(self, status: RequestStatus = RequestStatus.ALL) -> List[dict]:
        """
        Scrape access request data from the access request page for the specified request status.

        Data will be returned as a list of dicts of the form:
        ::
            {
                'user': 'Alex Student', # the requesting user's name
                'user_id': '1843927', # the requesting user's account ID
                'email': 'student.alex@gmail.com', # the requesting user's email
                'request_date': datetime.datetime(2019, 4, 12).date(), # the date the request was made
                'request': 'Student Access' # the request type, one of 'Student Access', 'Student Reactivation', or 'Mentor Access'],
                'status': 'waiting' # the request status, one of 'waiting', 'successful', 'rejected', or 'failed'
            }

        IMPORTANT: due to unknown limitations of Handshake's server, the request
        page appears to only be capable of loading up to around 1750 rows before
        crashing the website. To guard against this, this method will only pull
        the first 69 pages of results. If you have more than 1725 requests in any
        status, you will be unable to pull the entire dataset for that status.
        The method will only return the first 1725 rows.

        :param status: the status type of requests to pull. By default, pull data for all requests
        :type status: RequestStatus
        :return: a list of request data dicts
        :rtype: list
        """
        SEE_MORE_BTN_XPATH = "//a[text()='See More']"

        # filter page to only include requests of the given status
        self._browser.click_element_by_xpath(self._status_btn_xpath(status))
        self._browser.wait_until_element_is_clickable_by_xpath(
            "//*[@class='table manage-table']/tbody/tr[1]/td[1]")

        # load all requests of the given status onto the page
        total_requests = sum(map(int, self._browser.get_elements_attribute_by_xpath(
            "//div[@class='panel panel-default']//a/span[@class='badge' and text()!='0']", "text")))

        pages_to_load = int(math.floor(total_requests / 25))
        # account for apparent load limit on Handshake's servers
        if pages_to_load > 69:
            pages_to_load = 69
        for i in range(pages_to_load):  # click "See More" btn until all rows loaded
            self._browser.wait_until_element_exists_by_xpath(SEE_MORE_BTN_XPATH)
            self._browser.click_element_by_xpath(SEE_MORE_BTN_XPATH)

        time.sleep(2)  # wait for final set of names to finish loading before scraping

        # scrape request data from page
        request_data = []
        student_rows = BeautifulSoup(
            self._browser.get_element_attribute_by_xpath("//tbody", "innerHTML"),
            'html.parser').find_all('tr')
        for row in student_rows:
            user = row.find('td', {'class': 'requester_name'}).find('a').text
            user_id_link = row.find('td', {'class': 'requester_name'}).find('a')['href']
            user_id = re.findall(r'([0-9]+)', user_id_link)[0]
            date_str = row.find('td', {'class': 'request_date'}).find('a').text
            date_str = re.sub(r'(\d)(st|nd|rd|th)', r'\1', date_str)
            data_row = {
                'user': user,
                'user_id': user_id,
                'email': row.find('td', {'class': 'requester_email'}).find('a').text,
                'request_date': datetime.strptime(date_str, '%B %d %Y').date(),
                'request': row.find('td', {'class': 'request'}).find('a').text,
                'status': row.find('td', {'class': 'status'}).find('span').text
            }
            request_data.append(data_row)
        return request_data

    def _validate_url(self, url):
        """
        Ensure that the given URL is a valid URL.

        Since the URL is not entered by the user, it is always valid.

        :param url: the url to validate
        :type url: str
        """
        return

    def _wait_until_page_is_loaded(self):
        """Wait until the page has finished loading."""
        for status in RequestStatus:
            self._browser.wait_until_element_is_clickable_by_xpath(
                self._status_btn_xpath(status))

    @staticmethod
    def _status_btn_xpath(status: RequestStatus):
        """Get the xpath for the button to load request data for the given status"""
        return f'//div[@id="notifications-affix"]//a[./span[text()="{status.value}"]]'



from autohandshake.src.Pages.Page import Page
from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.constants import BASE_URL
from autohandshake.src.file_download_utils import confirm_file_downloaded
from datetime import datetime


class SurveyPage(Page):
    """
    A survey page in Handshake.
    """

    def __init__(self, survey_id: int, browser: HandshakeBrowser):
        """
        :param survey_id: the id of the survey to load
        :type survey_id: int
        :param browser: a logged-in HandshakeBrowser
        :type browser: HandshakeBrowser
        """
        self._id = survey_id
        super().__init__(f'{BASE_URL}/surveys/{survey_id}', browser)

    @Page.require_user_type(UserType.STAFF)
    def download_responses(self, download_dir: str, wait_time=300) -> str:
        """
        Download a CSV of the survey responses.

        :param download_dir: the directory into which the survey responses will download
        :type download_dir: str
        :param wait_time: the maximum time to wait for the download to appear
        :type wait_time: int
        :return: the file path of the downloaded file
        :rtype: str
        """
        download_btn_xpath = '//a[text()="Download Results (CSV)"]'
        download_link_xpath = '//a[contains(text(), "Your download is ready")]'
        self._browser.wait_then_click_element_by_xpath(download_btn_xpath)
        self._browser.wait_then_click_element_by_xpath(download_link_xpath)
        filename_pattern = self._get_filename_pattern()
        return confirm_file_downloaded(download_dir, filename_pattern, wait_time)

    @staticmethod
    def _get_filename_pattern() -> str:
        """
        Get a regex string describing the form of the downloaded file.
        """
        return f'survey_response_download{datetime.now().strftime("%Y%m%d")}*.csv'

    def _validate_url(self, url):
        """
        Ensure that the given URL is a valid URL.

        Since the URL is not entered by the user, it is always valid.

        :param url: the url to validate
        :type url: str
        """
        return

    def _wait_until_page_is_loaded(self):
        """Wait until the page has finished loading."""
        self._browser.wait_until_element_exists_by_xpath('//div[@class="entity-sidebar tile white"]')



from autohandshake.src.Pages import Page
from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.constants import BASE_URL
from autohandshake.src.exceptions import InvalidURLError, NoSuchElementError, \
    InsufficientPermissionsError
import re
import os
import time
from typing import List
import json
from enum import Enum
from datetime import date

DEFAULT_WAIT_TIME = 300


class FileType(Enum):
    """
    The possible download file types in Insights

            * EXCEL - an excel file with a .xlsx extension
            * TXT - a tab-separated .txt file
            * JSON - a json file with a .json extension
            * HTML - a .html file that can render the data table in a browser
            * MARKDOWN - a .md file that renders the table in markdown
            * PNG - a .png image of the data table
            * CSV - a comma-separated .csv file
    """
    EXCEL = 'xlsx'
    TXT = 'txt'
    JSON = 'json'
    JSON_LABEL = 'json_label'
    HTML = 'html'
    MARKDOWN = 'md'
    PNG = 'png'
    CSV = 'csv'


class _DownloadModal:
    """An interface to the pop-up modal for downloading Insights files"""

    def __init__(self, browser: HandshakeBrowser):
        """
        Open the download modal on the Insights page

        :param browser: a logged-in HandshakeBrowser on the Insights Page
        :type browser: HandshakeBrowser
        """
        self._browser = browser
        self.is_open = False

    @Page.require_user_type(UserType.STAFF)
    def open(self):
        """Open the insights report download modal box."""
        if not self.is_open:
            self._browser.wait_then_click_element_by_xpath("//i[@class='lk-icon-gear']")
            self._browser.wait_then_click_element_by_xpath("//a[@ng-click='openDownloadDialog()']")
            self._browser.wait_until_element_is_clickable_by_xpath(
                "//div[contains(@class, 'query-download-modal-limit')]")
            # click modal body to bring it into focus, allowing additional actions
            self._browser.click_element_by_xpath("//div[@class='modal-body']")
            self.is_open = True

    @Page.require_user_type(UserType.STAFF)
    def validate_download_modal_is_open(self):
        """Throw an error if the download modal is not open."""
        if not self.is_open:
            raise NoSuchElementError('Insights Download modal box must be open')

    @Page.require_user_type(UserType.STAFF)
    def get_download_file_type(self) -> FileType:
        """
        Get the name of the file type currently selected.

        :return: the name of the file type currently selected
        :rtype: FileType
        """
        self.validate_download_modal_is_open()
        value = self._browser.get_element_attribute_by_xpath(
            "//select[@id='qr-export-modal-format']", "value")
        # values are formatted "string:[value name]." we must remove the first 7 chars
        return FileType(value[7:])

    @Page.require_user_type(UserType.STAFF)
    def set_download_file_type(self, file_type: FileType):
        """
        If the download modal is open, select the file type to download.

        :param file_type: the name of a valid file type to download from insights
        :type file_type: FileType
        """
        self.validate_download_modal_is_open()
        type_label_lookup = {
            FileType.TXT: 'TXT (tab-separated values)',
            FileType.EXCEL: 'Excel Spreadsheet (Excel 2007 or later)',
            FileType.CSV: 'CSV',
            FileType.JSON: 'JSON',
            FileType.HTML: 'HTML',
            FileType.MARKDOWN: 'Markdown',
            FileType.PNG: 'PNG (Image of Visualization)'
        }
        try:
            type_label = type_label_lookup[file_type]
            self._browser.click_element_by_xpath(f"//option[@label='{type_label}']")
        except KeyError:
            raise ValueError(f'Invalid file type: "{file_type}"')

    @Page.require_user_type(UserType.STAFF)
    def set_limit_to_all_results(self, remove_sorts: bool = False):
        """
        If the download modal is open, set the download limit to "all results,"
        optionally removing sorts on the query for increased performance.

        :param remove_sorts: whether or not to remove sorts on the query
        :type remove_sorts: bool
        """
        self.validate_download_modal_is_open()
        all_results_radio_xpath = "//input[@name='qr-export-modal-limit' and @value='all']"
        remove_sorts_xpath = "//input[@ng-model='queryDownloadController.runUnsorted']"
        time.sleep(0.1)  # account for unknown, inconsistent timing error that prevents clicking
        try:
            self._browser.wait_until_element_is_clickable_by_xpath(all_results_radio_xpath)
            self._browser.click_element_by_xpath(all_results_radio_xpath)
            if remove_sorts:
                self._browser.wait_until_element_is_clickable_by_xpath(remove_sorts_xpath)
                self._browser.click_element_by_xpath(remove_sorts_xpath)
        except NoSuchElementError:
            raise InsufficientPermissionsError('You do not have permission '
                                               'to download all results')

    @Page.require_user_type(UserType.STAFF)
    def set_custom_limit(self, limit: int):
        """
        If the download modal is open, set the download limit to a custom number
        specified by "limit"

        :param limit: the custom download limit
        :type limit: int
        """
        self.validate_download_modal_is_open()
        custom_limit_radio_xpath = "//input[@name='qr-export-modal-limit' and @value='custom']"
        custom_limit_value_xpath = "//input[@name='customExportLimit']"
        time.sleep(0.1)  # account for unknown, inconsistent timing error that prevents clicking
        self._browser.wait_until_element_is_clickable_by_xpath(custom_limit_radio_xpath)
        self._browser.click_element_by_xpath(custom_limit_radio_xpath)
        self._browser.wait_until_element_is_clickable_by_xpath(custom_limit_value_xpath)
        self._browser.send_text_to_element_by_xpath(custom_limit_value_xpath, str(limit))

    @Page.require_user_type(UserType.STAFF)
    def get_file_name(self) -> str:
        """
        If the download modal is open, get the file name of the file to be
        downloaded.

        :returns: the name of the file to be downloaded
        :rtype: str
        """
        self.validate_download_modal_is_open()
        file_name_xpath = "//input[@name='customExportFilename']"
        self._browser.wait_until_element_exists_by_xpath(file_name_xpath)
        return self._browser.get_element_attribute_by_xpath(file_name_xpath, 'value')

    @Page.require_user_type(UserType.STAFF)
    def set_file_name(self, file_name: str):
        """
        If the download modal is open, set the file name of the file to be
        downloaded.

        :param file_name: the name of the file to be downloaded
        :type file_name: str
        """
        self.validate_download_modal_is_open()
        file_name_xpath = "//input[@name='customExportFilename']"
        if not file_name:
            raise ValueError('File name cannot be null or ""')
        self._browser.wait_until_element_exists_by_xpath(file_name_xpath)
        file_name = self.ensure_file_name_has_correct_extension(file_name)
        self._browser.send_text_to_element_by_xpath(file_name_xpath, file_name)

    @Page.require_user_type(UserType.STAFF)
    def click_download_button(self, download_dir: str, max_wait_time: int = DEFAULT_WAIT_TIME) -> str:
        """
        If the download modal is open, download the Insights report

        :param download_dir: the directory into which Insights will download the
                             file. This is used to confirm that the file
                             downloaded successfully, *not* to tell Insights
                             where to download the file (Insights has no power
                             over that).
        :type download_dir: str
        :param max_wait_time: the maximum amount of time to wait for the file to
                              download without throwing an error
        :type max_wait_time: int
        :return: the full file path of the downloaded file
        :rtype: str
        """
        self.validate_download_modal_is_open()
        file_name = self.get_file_name()
        file_path = os.path.join(download_dir, file_name)
        self._browser.click_element_by_xpath("//button[@id='qr-export-modal-download']")
        self._confirm_file_downloaded(file_path, max_wait_time)
        self.is_open = False
        return file_path

    @Page.require_user_type(UserType.STAFF)
    def click_open_in_browser(self) -> List[dict]:
        """
        If the download modal is open, open the data report in the browser in a
        new tab.

        :returns: a python list of dicts representing the data generated by Insights
        :rtype: list
        """
        valid_file_types = [FileType.JSON, FileType.JSON_LABEL]
        if not self.get_download_file_type() in valid_file_types:
            raise RuntimeError("Getting data in browser is only available for the JSON file type at this time")
        self.validate_download_modal_is_open()
        self._browser.click_element_by_xpath("//button[@id='qr-export-modal-open']")
        self._browser.switch_to_new_tab()
        self._browser.wait_until_element_exists_by_xpath("//pre")
        raw_text = self._browser.get_element_attribute_by_xpath("//pre", "text")
        self._browser.return_to_main_tab()
        self.is_open = False
        return json.loads(raw_text)

    @staticmethod
    def _confirm_file_downloaded(file_path, wait_time):
        """
        Confirm that the file at the given file path has downloaded successfully.

        Throw an error if the file still does not exists when the given wait time
        is up

        :param file_path: the expected file path of the downloading file
        :type file_path: str
        :param wait_time: the maximum time to wait for the file to download
        :type wait_time: integer
        """
        start_time = time.time()
        time_to_stop = start_time + wait_time
        while time.time() <= time_to_stop:
            time.sleep(0.5)
            if os.path.exists(file_path):
                return

        raise RuntimeError('File did not download in time')

    def ensure_file_name_has_correct_extension(self, file_name) -> str:
        """Make sure the file name has the correct extension for the selected file type"""
        file_type = self.get_download_file_type()
        if file_name[(-1) * len(str(file_type.value)):] != file_type.value:
            file_name = file_name + '.' + file_type.value
        return file_name


class InsightsPage(Page):
    """Handshake's custom data reporting page (powered by Looker)"""

    def __init__(self, url_string: str, browser: HandshakeBrowser):
        """
        :param url_string: either a full insights URL or just the alphanumeric
                           query string specifying the exact report
        :type url_string: str
        :param browser: a logged-in HandshakeBrowser
        :type browser: HandshakeBrowser
        """
        super().__init__(url_string, browser)
        if self._browser.element_exists_by_xpath(
                "//span[text()='Select some dimensions or measures.' and @aria-hidden='false']"):
            raise InvalidURLError('Insights URL has no dimensions or measures selected')
        self.modal = _DownloadModal(self._browser)

    @Page.require_user_type(UserType.STAFF)
    def get_data(self, limit: int = None) -> List[dict]:
        """Get a JSON-like list of dict representation of the Insights report's data

        :param limit: The maximum number of rows to retrieve. By default, there
                      is no limit.
        :type limit: int
        :returns: the Insight report's data in list-of-dict/JSON-like format
        :rtype: list
        """
        self.modal.open()
        self.modal.set_download_file_type(FileType.JSON)
        if limit is None:
            try:
                self.modal.set_limit_to_all_results(remove_sorts=True)
            except InsufficientPermissionsError:
                pass
                # keep limit at "Results in Table" if "All Results" is not available
        else:
            self.modal.set_custom_limit(limit)
        return self.modal.click_open_in_browser()

    @Page.require_user_type(UserType.STAFF)
    def download_file(self, download_dir: str, file_name: str = None, file_type: FileType = FileType.CSV,
                      max_wait_time: int = DEFAULT_WAIT_TIME, limit: int = None) -> str:
        """Download the Insights report's data in a file of the specified type

        :param file_type: the type of file to download
        :type file_type: FileType
        :param download_dir: the directory into which Insights will download the
                             file. This is used to confirm that the file
                             downloaded successfully, *not* to tell Insights
                             where to download the file (Insights has no power
                             over that).
        :type download_dir: str
        :param file_name: the name of the file to download. If None, default to
                          Handshake's standard naming of Insights files
        :type file_name: str
        :param max_wait_time: the maximum amount of time to wait for the file to
                              download without throwing an error
        :type max_wait_time: int
        :param limit: The maximum number of rows to retrieve. By default, there
                      is no limit.
        :type limit: int
        :returns: the filepath of the newly-downloaded file
        :rtype: str
        """
        self.modal.open()
        self.modal.set_download_file_type(file_type)
        if file_name:
            self.modal.set_file_name(file_name)
        if limit is None:
            try:
                self.modal.set_limit_to_all_results(remove_sorts=True)
            except InsufficientPermissionsError:
                pass
                # keep limit at "Results in Table" if "All Results" is not available
        else:
            self.modal.set_custom_limit(limit)
        return self.modal.click_download_button(download_dir, max_wait_time)

    @Page.require_user_type(UserType.STAFF)
    def set_date_range_filter(self, field_category: str, field_title: str,
                              start_date: date, end_date: date):
        """
        Set a date range filter on the Insights report to the given start and end date.

        Insights date filters are identified by a two-part name: the first part
        indicates the "category" or subject matter the date refers to, and the
        second part indicates the specific date field on that category.

        For example: the filter for the start date of an appointment is called
        "Appointments Start Date Date." In the Insights filter box, this title
        appears with the "Start Date Date" part in bold and the "Appointments"
        part in regular type. In this case "Appointments" is the field_category,
        and "Start Date Date" is the field_title. This methods requires both
        parts of the name in order to apply the given start and end dates to the
        correct filter.

        IMPORTANT: at this time, this method only works on date range filters
        that follow the form "[Category] [Title] is in range [start] until (before) [end]."
        The method does not work on month or time filters, or on filters that do
        not represent a range between two time-less dates. Also, the filter must
        already exist in the Insights page; this method will not create a filter,
        merely modify the dates of an existing one.

        :param field_category: the date filter's category name (see method description)
        :type field_category: str
        :param field_title: the date filter's title (see method description)
        :type field_title: str
        :param start_date: the start date to use in the filter
        :type start_date: date
        :param end_date: the end date to use in the filter. IMPORTANT: Handshake
                         date ranges go *up to but not including* the end date,
                         so a filter from 1/15/19 to 1/30/19 will NOT include any
                         data from 1/30/19, for example.
        :type end_date: date
        """
        self._browser.click_element_by_xpath('//lk-expandable-pane[contains(@class,"filter-pane")]')
        calendar_icon_xpath = '//tr[./td/div[./span[text()="{}"]][./strong[' \
                              'text()="{}"]]]//td[@class="clause-filter"]/span' \
                              '[{}]/span/a[contains(@class, "calendar-icon")]'
        start_calendar_xpath = calendar_icon_xpath.format(field_category, field_title, 2)
        end_calendar_xpath = calendar_icon_xpath.format(field_category, field_title, 3)

        self._select_calendar_date(start_calendar_xpath, start_date)
        self._select_calendar_date(end_calendar_xpath, end_date)

    def _validate_url(self, url):
        """
        Ensure that url string given is either a valid Insights URL or a valid
        Insights query string.

        :param url: the url to validate
        :type url: str
        """
        try:  # try full URL match
            re.match(f'^{BASE_URL}' + r'/analytics/explore_embed\?insights_page=[a-zA-Z0-9_]+(=){0,2}$', url) \
                .group(0)
        except AttributeError:
            try:  # try query parameter match
                re.match(r'^[a-zA-Z0-9_]+(=){0,2}$', url) \
                    .group(0)
            except AttributeError:
                raise InvalidURLError()

    def _wait_until_page_is_loaded(self):
        """Wait until the page has finished loading."""
        if self._browser.element_exists_by_xpath('//div[text()="Invalid report link"]'):
            raise InvalidURLError('Insights URL is invalid')
        self._switch_to_looker_iframe()

    def _switch_to_looker_iframe(self):
        """Bring the Looker iframe into focus on the insights page"""
        exists_once_iframe_in_focus = "//i[@class='lk-icon-gear']"
        if not self._browser.element_exists_by_xpath(exists_once_iframe_in_focus):
            if self._browser.element_exists_by_xpath("//div[@style='display: none']"):
                raise InvalidURLError("Insights query string is malformed")

            iframe_xpath = "//iframe[@id='explore-iframe']"
            self._browser.wait_until_element_exists_by_xpath(iframe_xpath)
            self._browser.switch_to_frame_by_xpath(iframe_xpath)

            if self._browser.element_exists_by_xpath("//div[@class='error-http-status-code']"):
                raise InvalidURLError("Insights query string is malformed")

            self._browser.wait_until_element_exists_by_xpath(exists_once_iframe_in_focus)

    def _select_calendar_date(self, calendar_xpath: str, select_date: date):
        """Select a date on an Insights page calendar widget."""
        MODAL_XPATH = '//ul[@template-url="template/datepicker/popup.html"]'
        HEADING_BTN_XPATH = f'{MODAL_XPATH}//button[@role="heading"]'
        YEAR_BTN_XPATH = f'{MODAL_XPATH}//button[./span[text()="{select_date.strftime("%Y")}"]]'
        MONTH_BTN_XPATH = f'{MODAL_XPATH}//button[./span[text()="{select_date.strftime("%B")}"]]'
        DAY_BTN_XPATH = f'{MODAL_XPATH}//button[./span[text()="{select_date.strftime("%d")}" and not(contains(@class, "text-muted"))]]'

        # open modal
        self._browser.click_element_by_xpath(calendar_xpath)
        self._browser.wait_until_element_is_clickable_by_xpath(MODAL_XPATH)

        # prepare to select date in the order of year-month-day
        self._browser.click_element_by_xpath(HEADING_BTN_XPATH)
        self._browser.click_element_by_xpath(HEADING_BTN_XPATH)

        # select the date
        self._browser.click_element_by_xpath(YEAR_BTN_XPATH)
        self._browser.click_element_by_xpath(MONTH_BTN_XPATH)
        self._browser.click_element_by_xpath(DAY_BTN_XPATH)



from autohandshake.src import HandshakeBrowser
from autohandshake.src.Pages.StudentProfilePage import StudentProfilePage
from autohandshake.src.constants import BASE_URL


class ViewAsStudent:
    """
    A sub-session in which the user logs in as a student.

    Should be used as a context manager. Example:
    ::

        with HandshakeSession(school_url, email) as browser:
            with ViewAsStudent(student_id):
                # do something
    """

    def __init__(self, student_id: int, browser: HandshakeBrowser, stay_on_page: bool = False):
        """

        :param student_id: the numeric Handshake id of the student to view as
        :type student_id: int
        :param browser: a logged-in Handshake browser with a STAFF user type
        :type browser: HandshakeBrowser
        :param stay_on_page: whether or not to stay on the same page when logging
                             back out of the "View as Student" session. If False,
                             navigate back to the Handshake homepage when the
                             session is over. Defaults to False.
        :type stay_on_page: bool
        """
        self._id = student_id
        self._browser = browser
        self._stay_on_page = stay_on_page

    def __enter__(self):
        """
        Log in as the specified student.
        """
        profile_page = StudentProfilePage(self._id, self._browser)
        profile_page.view_as_student()

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Stop viewing as the student and return to the career services view."""
        self._browser.click_element_by_xpath('//a[@href="/users/stop_viewing_as"]')
        self._browser.update_constants()
        if not self._stay_on_page:
            self._browser.get(BASE_URL)



from autohandshake.src.Pages import Page
from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.constants import BASE_URL
from typing import List
from bs4 import BeautifulSoup


class StaffPage(Page):

    def __init__(self, browser: HandshakeBrowser):
        """
        :param browser: a logged-in HandshakeBrowser
        :type browser: HandshakeBrowser
        """
        majors_base_url = f'{BASE_URL}/schools/{browser.school_id}/manage_users'
        url_query = '?page=1&per_page=5000&sort_direction=desc&sort_column=default'
        super().__init__(majors_base_url + url_query,
                         browser)

    @Page.require_user_type(UserType.STAFF)
    def get_staff_names(self) -> List[dict]:
        """
        Get a list of names of all the school's active career center staff accounts.

        :return: a list full staff names of the form ['Mary Jeffries', 'John Cabot']
        :rtype: list
        """
        name_cards = BeautifulSoup(self._browser.get_element_attribute_by_xpath(
            '//div[@data-hook="search-results"]', 'innerHTML'),
            'html.parser').find_all('h2')
        return [card.find('a').text for card in name_cards]

    def _validate_url(self, url):
        """
        Ensure that the given URL is a valid URL.

        Since the URL is not entered by the user, it is always valid.

        :param url: the url to validate
        :type url: str
        """
        return

    def _wait_until_page_is_loaded(self):
        """Wait until the page has finished loading."""
        self._browser.wait_until_element_exists_by_xpath("//div[@data-hook='search-results']")



from autohandshake.src.Pages.SchoolSettings.AppointmentTypesListPage import AppointmentTypesListPage
from autohandshake.src.Pages.SchoolSettings.AppointmentTypePage import AppointmentTypePage
from autohandshake.src.Pages.SchoolSettings.MajorSettingsPage import MajorSettingsPage
from autohandshake.src.Pages.SchoolSettings.LabelSettingsPage import LabelSettingsPage
from autohandshake.src.Pages.SchoolSettings.StaffPage import StaffPage



import unittest
from autohandshake import InterviewSchedulePage
from autohandshake.tests import TestSession


class TestAppointmentTypePage(unittest.TestCase):

    def test_get_schedule_contact_info_with_multiple_contacts(self):
        TEST_SCHEDULE_ID = 61382  # Bloomberg SWE Interviews, Fall 2018

        expected = [{'name': 'Bloomberg Engineering Campus Recruitment', 'email': 'fsdrecruitment@bloomberg.com'},
                    {'name': 'Catherine Malatack', 'email': 'cmalatack@bloomberg.net'},
                    {'name': 'Katie Golub', 'email': 'kgolub@bloomberg.com'}]

        with TestSession() as browser:
            schedule_page = InterviewSchedulePage(TEST_SCHEDULE_ID, browser)
            self.assertEqual(expected, schedule_page.get_contact_info())

    def test_get_schedule_contact_info_with_no_contacts(self):
        TEST_SCHEDULE_ID = 41258  # Fake Hop-Pop schedule with no contacts

        expected = []

        with TestSession() as browser:
            schedule_page = InterviewSchedulePage(TEST_SCHEDULE_ID, browser)
            self.assertEqual(expected, schedule_page.get_contact_info())

    def test_get_reserved_rooms(self):
        TEST_SCHEDULE_ID = 61382  # Bloomberg SWE Interviews, Fall 2018

        expected = 7

        with TestSession() as browser:
            schedule_page = InterviewSchedulePage(TEST_SCHEDULE_ID, browser)
            self.assertEqual(expected, schedule_page.get_reserved_rooms())



import unittest
from autohandshake.src.Pages import AppointmentTypesListPage
from autohandshake.tests import TestSession


class TestAppointmentTypesListPage(unittest.TestCase):
    maxDiff = None

    def test_get_type_ids(self):
        with TestSession() as browser:
            types_page = AppointmentTypesListPage(browser)
            type_ids = types_page.get_type_ids()
            self.assertEqual(20930, type_ids[0])
            self.assertEqual(18406, type_ids[2])
            self.assertEqual(20932, type_ids[7])

    def test_get_type_settings(self):
        expected = [
            {
                'id': 20930,
                'name': 'Carey: Networking (30 Minutes)',
                'description': 'Strategize your approach to building your professional network and polish your networking skills.',
                'length': 30,
                'categories': ['Find Jobs/Internships/Opportunities'],
                'drop_in_enabled': True,
                'pre_message': 'Due to our staff size, we have a strict no-show policy for both in-person and virtual appointments. Please plan to arrive five minutes prior to your scheduled appointment.  If you cannot make your appointment, please cancel as soon as possible. You may cancel your appointment in Handshake up to 12 hours before your appointment date.  After 12 hours, you must contact our office at carey.careerdevelopment@jhu.edu or 410-234-9270 to cancel. For appointments at the Harbor East location, enter through the Carey Business School doors and go to the front desk. Explain that you have a coaching appointment and give the attendant the name of your Career Coach.  For appointments at the DC location, go to the Bernstein Offit Building (BOB) 1717 Massachusetts Ave, 8th floor.  We do not validate parking at the DC or Harbor East locations. In addition to the school\'s garage parking, local garages and street parking are available.',
                'pre_survey': 'Carey: Pre Appointment Survey',
                'post_message': 'To help us better assist you in the future, please take a moment to complete the attached brief survey.  We also want to celebrate your success and tailor career search strategies unique to your experience and needs. Please be sure to update us on your job interviews or offers by sending an email to carey.careerdevelopment@jhu.edu.',
                'post_survey': 'Carey: Post Appointment Survey',
                'staff_survey': None,
                'school_years': None,
                'cum_gpa_required': False,
                'cum_gpa': None,
                'major_groups': None,
                'colleges': None,
                'labels': ['system gen: dual degree cbs', 'temp label: dual degree cbs', 'system gen: cbs'],
                'career_clusters': None
            },
            {
                'id': 9022,
                'name': 'Homewood: Consulting - Internships (Paid & Unpaid) Search Strategies',
                'description': 'Devise a plan to search for your internship opportunities and use them to experience the professional world, build a network and expand your skills.',
                'length': 30,
                'categories': ['Find Jobs/Internships/Opportunities'],
                'drop_in_enabled': False,
                'pre_message': """Thank you for making an appointment with the Johns Hopkins Career Center.  

To make the most of your 30 minute appointment, be prepared to share your main goals for your appointment with your coach.

To learn more about the ethical standards (www.naceweb.org/career-development/organizational-structure/principles-for-ethical-professional-practice/) and professional principles (www.naceweb.org/career-development/organizational-structure/principles-for-professional-practice/#careerservices) that guide our career coaching practice, visit the websites listed. 

Please note where your appointment is happening (Career Center, virtually, etc.). If you are unable to attend your appointment or will be more than 10 minutes late, please call the Career Center 410/516-8056 to cancel your appointment.  

Appointment times are held for 10 minutes only. After that time, you will be labeled as a “no show” in Handshake. After 3 “no-shows” in one fiscal year (July 1-June 30), you will lose the ability to make another appointment for the remainder of the fiscal year. 

Feel free to contact the Career Center with any questions.

We look forward to speaking with you!

Johns Hopkins Career Center
Garland Hall, Suite 389
Tel: 410-516-8056
career@jhu.edu\n""",
                'pre_survey': None,
                'post_message': 'The Career Center is dedicated to improving the services it offers students including individual coaching sessions. Your input is very important to this process. Thank you for completing a short 4-question survey on your recent coaching session. Please contact the Career Center if you have any questions at career@jhu.edu.',
                'post_survey': 'Homewood: 2017 Internship Search post-appointment survey',
                'staff_survey': 'Homewood: Post-Appointment Staff Survey',
                'school_years': ['Sophomore', 'Junior', 'Senior'],
                'cum_gpa_required': False,
                'cum_gpa': None,
                'major_groups': None,
                'colleges': None,
                'labels': ['hwd: old appointment type'],
                'career_clusters': ['Homewood: Consulting']
            }
        ]

        with TestSession() as browser:
            types_page = AppointmentTypesListPage(browser)
            type_settings = types_page.get_type_settings(how_many=2)
            self.assertListEqual(expected, type_settings)



import unittest
from autohandshake.src.Pages import LoginPage
from autohandshake.tests import email, password, homepage
from autohandshake import HandshakeBrowser
from autohandshake.src import InvalidURLError, InvalidEmailError, \
    InvalidPasswordError


class TestLoginPage(unittest.TestCase):

    def test_throws_error_if_given_invalid_url(self):
        misspelled_school_url = "https://jhuu.joinhandshake.com/login"
        invalid_url = f"{homepage}/events"

        with self.assertRaises(InvalidURLError):
            login_page = LoginPage(misspelled_school_url, HandshakeBrowser())
        with self.assertRaises(InvalidURLError):
            login_page = LoginPage(invalid_url, HandshakeBrowser())

    def test_doesnt_throw_error_if_given_valid_url(self):
        valid_url = homepage
        valid_url_with_login = f"{homepage}/login"

        try:
            login_page = LoginPage(valid_url, HandshakeBrowser())
        except:
            self.fail()

        try:
            login_page = LoginPage(valid_url_with_login, HandshakeBrowser())
        except:
            self.fail()

    def test_login_works_with_valid_credentials(self):
        browser = HandshakeBrowser()
        login_page = LoginPage(homepage, browser)
        login_page.login(email, password)
        browser.quit()

    def test_login_throws_exception_for_invalid_email(self):
        fake_email = "f@ke_email@jhu.edu"
        fake_pw = ""

        browser = HandshakeBrowser()
        login_page = LoginPage(homepage, browser)
        with self.assertRaises(InvalidEmailError):
            login_page.login(fake_email, fake_pw)
            browser.quit()

    def test_login_throws_exception_for_invalid_password(self):
        fake_pw = "not a password"

        browser = HandshakeBrowser()
        login_page = LoginPage(homepage, browser)
        with self.assertRaises(InvalidPasswordError):
            login_page.login(email, fake_pw)
            browser.quit()



import unittest

from autohandshake import EventsPage
from autohandshake.tests import TestSession
import csv


def csv_to_list_of_dicts(filepath):
    with open(filepath, encoding='utf-8') as file:
        return [{k: v for k, v in row.items()} for row in csv.DictReader(file, skipinitialspace=True)]


class TestEventsPage(unittest.TestCase):

    def test_load_saved_search(self):
        other_checkbox_xpath = '//input[@id="5_event_types"]'
        start_date_xpath = '//input[@id="start_date"]'
        end_date_xpath = '//input[@id="end_date"]'

        with TestSession() as browser:
            page = EventsPage(browser)
            self.assertEqual('', page._browser.get_element_attribute_by_xpath(start_date_xpath, 'value'))
            self.assertEqual('', page._browser.get_element_attribute_by_xpath(end_date_xpath, 'value'))

            # test valid search
            page.load_saved_search('Test Saved Search')
            self.assertEqual(True, page._browser.element_is_selected_by_xpath(other_checkbox_xpath))
            self.assertEqual('2017-08-29', page._browser.get_element_attribute_by_xpath(start_date_xpath, 'value'))
            self.assertEqual('2017-08-31', page._browser.get_element_attribute_by_xpath(end_date_xpath, 'value'))

            # test invalid search
            with self.assertRaises(ValueError):
                page.load_saved_search('This isnt a saved search')

    def test_download_event_data(self):
        with TestSession() as browser:
            page = EventsPage(browser)
            page.load_saved_search('Test Saved Search')
            filepath = page.download_event_data(TestSession.download_dir)
            actual = csv_to_list_of_dicts(filepath)
            self.assertEqual(2, len(actual))
            self.assertEqual('69611', actual[0]['Id'])
            self.assertEqual('78187', actual[1]['Id'])



import time
import os
import glob
from typing import List


def confirm_file_downloaded(dir: str, filename_pattern: str, wait_time: int = 300) -> str:
    """
    Confirm that a new file matching the filename_pattern downloaded successfully.

    :param dir: the expected download directory for the file
    :type dir: str
    :param filename_pattern: a regex pattern describing the newly-downloaded file
    :type filename_pattern: str
    :param wait_time: the maximum time to wait for the file to download
    :type wait_time: integer
    :return: the file path of the newly downloaded file
    :rtype: str
    """
    baseline_matches = _get_filename_matches(filename_pattern, dir)
    start_time = time.time()
    time_to_stop = start_time + wait_time
    while time.time() <= time_to_stop:
        time.sleep(0.5)
        updated_matches = _get_filename_matches(filename_pattern, dir)
        if len(updated_matches) == len(baseline_matches) + 1:
            return max(updated_matches, key=os.path.getctime)
    raise RuntimeError('File did not download in time')


def _get_filename_matches(filename_pattern: str, dir: str) -> List[str]:
    return glob.glob(os.path.join(dir, filename_pattern))



import unittest
import datetime
from autohandshake import WaitingRoomPage
from bs4 import BeautifulSoup


class TestWaitingRoomPageParser(unittest.TestCase):

    def test_parse_entry_with_office_location_not_being_created(self):
        check_in_soup = BeautifulSoup("""
        <div class="list-group-item clearfix">
            <div class="pull-right">
                <div>
                    <a class="btn btn-primary"
                        data-bind="click: $parent.check_drop_in_status_and_act, css: { 'btn-primary': status() == 'pending', 'btn-warning': status() != 'pending' }, text: status() == 'pending' ? 'Start Appointment' : 'Being Created'"
                        href="#">Start Appointment</a>
                </div>
                <div class="margin-top center">
                    <a class="text-danger" data-bind="click: $parent.remove_drop_in" href="#">Cancel Drop-in</a>
                </div>
            </div>
            <h3 class="no-margin-top">
                <a data-bind="text: fancy_title, attr: { href: '/users/' + user_id() }" target="_blank"
                    href="/users/8766322">Rachael Gamlin</a>
            </h3>
            <h4 data-bind="text: 'Checked In: ' + Handshake.Helpers.fullStraightDateTime(created_at())">Checked In: Saturday,
                September 22nd 2018 12:24 pm</h4>
            <h4 data-bind="visible: office_location_name(), text: 'Office Location: ' + office_location_name()">Office Location: Homewood AMS Career Connections - 100 Whitehead Hall</h4>
        </div>
        """, 'html.parser')
        expected = {
            'being_created': False,
            'student_id': 8766322,
            'student_name': 'Rachael Gamlin',
            'datetime': datetime.datetime(2018, 9, 22, 12, 24),
            'office_location': 'Homewood AMS Career Connections - 100 Whitehead Hall'
        }

        self.assertEqual(expected, WaitingRoomPage._parse_checkin(check_in_soup))

    def test_parse_entry_with_office_location_being_created(self):
        check_in_soup = BeautifulSoup("""
        <div class="list-group-item clearfix">
            <div class="pull-right">
                <div>
                    <a class="btn btn-warning"
                        data-bind="click: $parent.check_drop_in_status_and_act, css: { 'btn-primary': status() == 'pending', 'btn-warning': status() != 'pending' }, text: status() == 'pending' ? 'Start Appointment' : 'Being Created'"
                        href="#">Being Created</a>
                </div>
                <div class="margin-top center">
                    <a class="text-danger" data-bind="click: $parent.remove_drop_in" href="#">Cancel Drop-in</a>
                </div>
            </div>
            <h3 class="no-margin-top">
                <a data-bind="text: fancy_title, attr: { href: '/users/' + user_id() }" target="_blank"
                    href="/users/1468803">Jon Silveira</a>
            </h3>
            <h4 data-bind="text: 'Checked In: ' + Handshake.Helpers.fullStraightDateTime(created_at())">Checked In: Wednesday,
                October 31st 2018 2:33 pm</h4>
            <h4 data-bind="visible: office_location_name(), text: 'Office Location: ' + office_location_name()"
                style="display: none;">Office Location: null</h4>
        </div>
        """, 'html.parser')
        expected = {
            'being_created': True,
            'student_id': 1468803,
            'student_name': 'Jon Silveira',
            'datetime': datetime.datetime(2018, 10, 31, 14, 33),
            'office_location': None
        }

        self.assertEqual(expected, WaitingRoomPage._parse_checkin(check_in_soup))



import unittest
from autohandshake.src.Pages import MajorSettingsPage
from autohandshake.tests import TestSession


class TestMajorSettingsPage(unittest.TestCase):

    def test_get_major_mapping(self):
        with TestSession() as browser:
            major_settings_page = MajorSettingsPage(browser)
            mapping = major_settings_page.get_major_mapping()
            self.assertEqual('AD: Piano', mapping[4]['major'])
            self.assertEqual(['Music & Music Education', 'Visual & Performing Arts'], sorted(mapping[4]['groups']))

            self.assertEqual('Applied Physics', mapping[14]['major'])
            self.assertEqual(['Aerospace Engineering', 'General Engineering', 'Physics'], sorted(mapping[14]['groups']))

            self.assertEqual('Bachelors: Film & Media Studies', mapping[40]['major'])
            self.assertEqual(['Documentary/Film', 'Public Relations', 'Radio, Television, Media'],
                             sorted(mapping[40]['groups']))
